/* --------------------------------------------------------------------
	Project: SOKOBAN GAME
	Module:  SOK.C
	Author:  AL SCHOEPP
	Started: July 15, 1995
	Subject: Sokoban game which allows user to create levels
	-------------------------------------------------------------------- */

/* --------------------------------------------------------------------
						includes
   -------------------------------------------------------------------- */
#include	"hp-sok.h"
#define ROW_DISP	-2

/* --------------------------------------------------------------------
						global variables
   -------------------------------------------------------------------- */
extern unsigned char WallBlock[];
extern unsigned char TopWallBlock[];
extern unsigned char BottomWallBlock[];
extern unsigned char Pusher[];
extern unsigned char PusherSafe[];
extern unsigned char SafeSpot[];
extern unsigned char BlankSpot[];
extern unsigned char Block[];
extern unsigned char BlockSafe[];
extern unsigned char SokPCX[];

FILE *fp;

char PRow;				/*pusher's current row*/
char PCol;				/*pusher's current column*/
char NumSafe;			/*number of blocks safe*/
char Blocks;			/*number of blocks*/
char Num_Levels;		/*number of levels in game*/
char level_done;
char level;

char last_move;

char MazeArray[MAX_COLS][MAX_ROWS];
char SokDisk[MAX_COLS * MAX_ROWS / 2];

/* define the function key labels */
char *KeyLabels[10] = {
	"Help", "Restart", "Undo", "Prev", "Next",
	"Go to", NULL,     "Quit",  NULL,   NULL
};


/* the 'File' pulldown: Items and Menu */
MENUITEM FileItems[] = {
	{ " &Restart   F2 ", MA_VAL,  MENUVAL(F2_KEY)   },
	{ " &Undo      F3 ", MA_VAL,  MENUVAL(F3_KEY)   },
	{ " &Previous  F4 ", MA_VAL,  MENUVAL(F4_KEY)	},
	{ " &Next      F5 ", MA_VAL,  MENUVAL(F5_KEY)	},
	{ " &Go to     F6 ", MA_VAL,  MENUVAL(F6_KEY)   },
};
MENU FileMenu = { NULL, MS_PULLDN, 0, 0, 5, 5, FileItems };


/* the 'Help' pulldown: Items and Menu */
MENUITEM HelpItems[] = {
	{ " &Help   F1 ",	MA_VAL,    MENUVAL(F1_KEY)      },
	{ " &About ",		MA_VAL,    MENUVAL(CMD_ABOUT)   },
};
MENU HelpMenu = { NULL, MS_PULLDN, 0, 0, 2, 2, HelpItems };

/* The menu bar itself, branching off to the submenues */
MENUITEM BarItems[] = {
	{ "&Level",	MA_MNU,	&FileMenu		},
	{ "&Quit",	MA_VAL, MENUVAL(F8_KEY)	},
	{ "&Help",	MA_MNU, &HelpMenu		},
};
MENU BarMenu = { NULL, MS_HORIZ|MS_TOPLVL, 0, 0, 3, 3, BarItems };


/* --------------------------------------------------------------------
						   functions
   -------------------------------------------------------------------- */

/*
 *	go to next level
 */
void NextLevel(char PrevNext)
{
	int len, len2, i, n, r, c;
	char temp, t;
	char string[4];

	switch(PrevNext){
		case PREV:
			if((level - 1) < 1){
				return;
			}

			level--;
			fseek(fp, level * sizeof(SokDisk), SEEK_SET);
			break;
		case NEXT:
			if((level) >= Num_Levels){
				return;
			}

			level++;
			fseek(fp, level * sizeof(SokDisk), SEEK_SET);
			break;
		case NEXT_DONE:
			fseek(fp, 0L, SEEK_SET);
			fread(SokDisk, sizeof(SokDisk), 1, fp);
			Num_Levels = SokDisk[0];

			for(i = 1; i < Num_Levels; i++){
				if(SokDisk[i] == NOT_DONE)
					break;
			}

			level = i;
			if ((level == Num_Levels) && (SokDisk[level] == DONE)){
				Win(FINISHED);
			}

			fseek(fp, level * sizeof(SokDisk), SEEK_SET);
			break;
		case SELECT:
			t = SelectLevel();
			if(t == 0)
				return;
			level = t;
			fseek(fp, level * sizeof(SokDisk), SEEK_SET);
			break;
		case RESTART:
			break;
	}

	if(PrevNext  != RESTART)
		fread(SokDisk, sizeof(SokDisk), 1, fp);

	itoa(level, string, 10);
	len = TextExt(MEDIUM_FONT, string);
	len2 = TextExt(MEDIUM_FONT, "LEVEL");
	TextOut(570, 45, TXT_RULE, MEDIUM_FONT, "     ");
	TextOut(570 + ((len2 - len) / 2), 45, TXT_RULE, MEDIUM_FONT, string);

	NumSafe = 0;
	Blocks = 0;

	for(i = 0; i < SokDisksize; i++){

		for(n = 0; n < 2; n++){

			r = (2 * i + n) / MAX_COLS;
			c = (2 * i + n) - (r * MAX_COLS);

			if(!(n & 1)){
				temp = SokDisk[i];
				MazeArray[c][r] = temp >>= 4;
			}else{
				MazeArray[c][r] = SokDisk[i] & 0x0F;
			}

			if(r == 0){
				PutImg(c * BLOCK_WIDTH, 0, FORCE_RULE, BottomWallBlock);
				MazeArray[c][r] = WALL_BLOCK;
			}else if(r == 12){
				PutImg(c * BLOCK_WIDTH, 178, FORCE_RULE, TopWallBlock);
				MazeArray[c][r] = WALL_BLOCK;
			}else{
				switch(MazeArray[c][r]){
					case BLOCK:
						PutImg(c * BLOCK_WIDTH, r * BLOCK_HEIGHT + ROW_DISP, FORCE_RULE, Block);
						Blocks++;
						MazeArray[c][r] = BLOCK;
						break;
					case BLOCK_SAFE:
						PutImg(c * BLOCK_WIDTH, r * BLOCK_HEIGHT + ROW_DISP, FORCE_RULE, BlockSafe);
						MazeArray[c][r] = BLOCK_SAFE;
						NumSafe++;
						Blocks++;
						break;
					case WALL_BLOCK:
						PutImg(c * BLOCK_WIDTH, r * BLOCK_HEIGHT + ROW_DISP, FORCE_RULE, WallBlock);
						MazeArray[c][r] = WALL_BLOCK;
						break;
					case PUSHER:
						PutImg(c * BLOCK_WIDTH, r * BLOCK_HEIGHT + ROW_DISP, FORCE_RULE, Pusher);
						MazeArray[c][r] = PUSHER;
						PRow = r;
						PCol = c;
						break;
					case PUSHER_SAFE:
						PutImg(c * BLOCK_WIDTH, r * BLOCK_HEIGHT + ROW_DISP, FORCE_RULE, PusherSafe);
						MazeArray[c][r] = PUSHER_SAFE;
						PRow = r;
						PCol = c;
						break;
					case SAFE_SPOT:
						PutImg(c * BLOCK_WIDTH, r * BLOCK_HEIGHT + ROW_DISP, FORCE_RULE, SafeSpot);
						MazeArray[c][r] = SAFE_SPOT;
						break;
					case BLANK_SPOT:
						PutImg(c * BLOCK_WIDTH, r * BLOCK_HEIGHT + ROW_DISP, FORCE_RULE, BlankSpot);
						MazeArray[c][r] = BLANK_SPOT;
						break;
					default:
						break;
				}
			}

		}
	}
}

/*
 *	select a level in the game
 */
int SelectLevel(void)
{
	int r, i;
	char buffer[80];

	do {
		r = MsgBox("Go To", "Enter the desired level.|          1-%i|Level:|", buffer, "  OK  [F10]|Cancel[F9]", Num_Levels);
		i = atoi(buffer);

		if((i < 1 || i > Num_Levels) && (r == 0 || r == DN_OK))
			putch('\a');

		if(r == 1 || r == DN_CANCEL)
			break;

	} while ((i < 1 || i > Num_Levels) && (r != 0 || r != DN_OK));

	if(r == 0 || r == DN_OK){
		return(i);
	}
	else{
		return(0);
	}
}





/*
 *	win a level or the game
 */
void Win(char AllDone)
{
	PALWIN	*pWin;

	pWin = OpenWin(WS_HP200, 120, 20, 515, 115, "");

	if (AllDone == NOT_FINISHED){
		TextOut(185, 40, TXT_RULE, LARGE_FONT, "Congratulations!!");
		TextOut(140, 60, TXT_RULE, MEDIUM_FONT, "You have made it past another level!");
		TextOut(140, 100, TXT_RULE, SMALL_FONT, "Press any key to continue...");
		GetKey();
	}
	else if (AllDone == FINISHED){
		TextOut(185, 40, TXT_RULE, LARGE_FONT, "Congratulations!!");

		TextOut(180, 60, TXT_RULE, MEDIUM_FONT, "You have made it to the end!");
		TextOut(195, 75, TXT_RULE, MEDIUM_FONT, "All levels are completed!");

		TextOut(140, 100, TXT_RULE, SMALL_FONT, "Press any key to reset game.");
		GetKey();

		memset(SokDisk, 0, 320 * sizeof(char));

		SokDisk[0] = Num_Levels;

		fseek(fp, 0L, SEEK_SET);
		fwrite(SokDisk, sizeof(SokDisk), 1, fp);

		CloseWin(pWin);
		NextLevel(NEXT_DONE);

		return;
	}


	CloseWin(pWin);

	level_done++;

	fseek(fp, 0L, SEEK_SET);
	fread(SokDisk, sizeof(SokDisk), 1, fp);
	fseek(fp, 0L, SEEK_SET);
	SokDisk[level] = DONE;

	fwrite(&SokDisk, sizeof(SokDisk), 1, fp);

	NextLevel(NEXT_DONE);

}


/*
 *	moves pusher in the appropriate direction
 */
void MoveChar(int Direction)
{
	char legal_move;

	switch(Direction){
		case UNDO:

			switch(last_move){
				case 10:
					if(MazeArray[PCol][PRow] == PUSHER_SAFE){
						PutImg(PCol * BLOCK_WIDTH, PRow * BLOCK_HEIGHT + ROW_DISP, FORCE_RULE, SafeSpot);
                        MazeArray[PCol][PRow] = SAFE_SPOT;
                    }
					else{
						PutImg(PCol * BLOCK_WIDTH, PRow * BLOCK_HEIGHT + ROW_DISP, FORCE_RULE, BlankSpot);
						MazeArray[PCol][PRow] = BLANK_SPOT;
					}

					PRow--;

					if(MazeArray[PCol][PRow] == SAFE_SPOT){
                        MazeArray[PCol][PRow] = PUSHER_SAFE;
						PutImg(PCol * BLOCK_WIDTH, PRow * BLOCK_HEIGHT + ROW_DISP, FORCE_RULE, PusherSafe);
					}
					else{
                        MazeArray[PCol][PRow] = PUSHER;
                        PutImg(PCol * BLOCK_WIDTH, PRow * BLOCK_HEIGHT + ROW_DISP, FORCE_RULE, Pusher);
					}

					break;
				case 11:
					if(MazeArray[PCol][PRow] == PUSHER_SAFE){
						MazeArray[PCol][PRow] = BLOCK_SAFE;
						PutImg(PCol * BLOCK_WIDTH, PRow * BLOCK_HEIGHT + ROW_DISP, FORCE_RULE, BlockSafe);
                    }
					else{
						MazeArray[PCol][PRow] = BLOCK;
                        PutImg(PCol * BLOCK_WIDTH, PRow * BLOCK_HEIGHT + ROW_DISP, FORCE_RULE, Block);
                    }

					if(MazeArray[PCol][PRow + 1] == BLOCK_SAFE){
						MazeArray[PCol][PRow + 1] = SAFE_SPOT;
						PutImg(PCol * BLOCK_WIDTH, (PRow + 1) * BLOCK_HEIGHT + ROW_DISP, FORCE_RULE, SafeSpot);
					}
					else{
                        MazeArray[PCol][PRow + 1] = BLANK_SPOT;
						PutImg(PCol * BLOCK_WIDTH, (PRow + 1) * BLOCK_HEIGHT + ROW_DISP, FORCE_RULE, BlankSpot);
                    }


					PRow--;

			        if(MazeArray[PCol][PRow] == SAFE_SPOT){
						MazeArray[PCol][PRow] = PUSHER_SAFE;
						PutImg(PCol * BLOCK_WIDTH, PRow * BLOCK_HEIGHT + ROW_DISP, FORCE_RULE, PusherSafe);
					}
					else{
						MazeArray[PCol][PRow] = PUSHER;
                        PutImg(PCol * BLOCK_WIDTH, PRow * BLOCK_HEIGHT + ROW_DISP, FORCE_RULE, Pusher);
					}

					break;
				case 20:
                    if(MazeArray[PCol][PRow] == PUSHER_SAFE){
                        PutImg(PCol * BLOCK_WIDTH, PRow * BLOCK_HEIGHT + ROW_DISP, FORCE_RULE, SafeSpot);
                        MazeArray[PCol][PRow] = SAFE_SPOT;
					}
					else{
						PutImg(PCol * BLOCK_WIDTH, PRow * BLOCK_HEIGHT + ROW_DISP, FORCE_RULE, BlankSpot);
						MazeArray[PCol][PRow] = BLANK_SPOT;
					}

					PRow++;

					if(MazeArray[PCol][PRow] == SAFE_SPOT){
						MazeArray[PCol][PRow] = PUSHER_SAFE;
                        PutImg(PCol * BLOCK_WIDTH, PRow * BLOCK_HEIGHT + ROW_DISP, FORCE_RULE, PusherSafe);
                    }
					else{
						MazeArray[PCol][PRow] = PUSHER;
						PutImg(PCol * BLOCK_WIDTH, PRow * BLOCK_HEIGHT + ROW_DISP, FORCE_RULE, Pusher);
					}

					break;
                case 21:
					if(MazeArray[PCol][PRow] == PUSHER_SAFE){
						MazeArray[PCol][PRow] = BLOCK_SAFE;
						PutImg(PCol * BLOCK_WIDTH, PRow * BLOCK_HEIGHT + ROW_DISP, FORCE_RULE, BlockSafe);
					}
                    else{
                        MazeArray[PCol][PRow] = BLOCK;
                        PutImg(PCol * BLOCK_WIDTH, PRow * BLOCK_HEIGHT + ROW_DISP, FORCE_RULE, Block);
					}

					if(MazeArray[PCol][PRow - 1] == BLOCK_SAFE){
						MazeArray[PCol][PRow - 1] = SAFE_SPOT;
						PutImg(PCol * BLOCK_WIDTH, (PRow - 1) * BLOCK_HEIGHT + ROW_DISP, FORCE_RULE, SafeSpot);
                    }
					else{
						MazeArray[PCol][PRow - 1] = BLANK_SPOT;
						PutImg(PCol * BLOCK_WIDTH, (PRow - 1) * BLOCK_HEIGHT + ROW_DISP, FORCE_RULE, BlankSpot);
					}

                    PRow++;

					if(MazeArray[PCol][PRow] == SAFE_SPOT){
						MazeArray[PCol][PRow] = PUSHER_SAFE;
						PutImg(PCol * BLOCK_WIDTH, PRow * BLOCK_HEIGHT + ROW_DISP, FORCE_RULE, PusherSafe);
					}
					else{
                        MazeArray[PCol][PRow] = PUSHER;
						PutImg(PCol * BLOCK_WIDTH, PRow * BLOCK_HEIGHT + ROW_DISP, FORCE_RULE, Pusher);
                    }

					break;
				case 30:
                    if(MazeArray[PCol][PRow] == PUSHER_SAFE){
                        PutImg(PCol * BLOCK_WIDTH, PRow * BLOCK_HEIGHT + ROW_DISP, FORCE_RULE, SafeSpot);
						MazeArray[PCol][PRow] = SAFE_SPOT;
					}
					else{
						PutImg(PCol * BLOCK_WIDTH, PRow * BLOCK_HEIGHT + ROW_DISP, FORCE_RULE, BlankSpot);
						MazeArray[PCol][PRow] = BLANK_SPOT;
                    }

                    PCol++;

					if(MazeArray[PCol][PRow] == SAFE_SPOT){
						MazeArray[PCol][PRow] = PUSHER_SAFE;
                        PutImg(PCol * BLOCK_WIDTH, PRow * BLOCK_HEIGHT + ROW_DISP, FORCE_RULE, PusherSafe);
                    }
					else{
						MazeArray[PCol][PRow] = PUSHER;
						PutImg(PCol * BLOCK_WIDTH, PRow * BLOCK_HEIGHT + ROW_DISP, FORCE_RULE, Pusher);
					}

                    break;
				case 31:
                    if(MazeArray[PCol][PRow] == PUSHER_SAFE){
						MazeArray[PCol][PRow] = BLOCK_SAFE;
						PutImg(PCol * BLOCK_WIDTH, PRow * BLOCK_HEIGHT + ROW_DISP, FORCE_RULE, BlockSafe);
                    }
                    else{
                        MazeArray[PCol][PRow] = BLOCK;
						PutImg(PCol * BLOCK_WIDTH, PRow * BLOCK_HEIGHT + ROW_DISP, FORCE_RULE, Block);
					}

					if(MazeArray[PCol - 1][PRow] == BLOCK_SAFE){
						MazeArray[PCol - 1][PRow] = SAFE_SPOT;
                        PutImg((PCol - 1) * BLOCK_WIDTH, PRow * BLOCK_HEIGHT + ROW_DISP, FORCE_RULE, SafeSpot);
					}
                    else{
						MazeArray[PCol - 1][PRow] = BLANK_SPOT;
						PutImg((PCol - 1) * BLOCK_WIDTH, PRow * BLOCK_HEIGHT + ROW_DISP, FORCE_RULE, BlankSpot);
                    }

                    PCol++;

					if(MazeArray[PCol][PRow] == SAFE_SPOT){
						MazeArray[PCol][PRow] = PUSHER_SAFE;
						PutImg(PCol * BLOCK_WIDTH, PRow * BLOCK_HEIGHT + ROW_DISP, FORCE_RULE, PusherSafe);
					}
                    else{
						MazeArray[PCol][PRow] = PUSHER;
                        PutImg(PCol * BLOCK_WIDTH, PRow * BLOCK_HEIGHT + ROW_DISP, FORCE_RULE, Pusher);
					}

                    break;
				case 40:
                    if(MazeArray[PCol][PRow] == PUSHER_SAFE){
						PutImg(PCol * BLOCK_WIDTH, PRow * BLOCK_HEIGHT + ROW_DISP, FORCE_RULE, SafeSpot);
						MazeArray[PCol][PRow] = SAFE_SPOT;
					}
					else{
						PutImg(PCol * BLOCK_WIDTH, PRow * BLOCK_HEIGHT + ROW_DISP, FORCE_RULE, BlankSpot);
						MazeArray[PCol][PRow] = BLANK_SPOT;
					}

					PCol--;

					if(MazeArray[PCol][PRow] == SAFE_SPOT){
                        MazeArray[PCol][PRow] = PUSHER_SAFE;
                        PutImg(PCol * BLOCK_WIDTH, PRow * BLOCK_HEIGHT + ROW_DISP, FORCE_RULE, PusherSafe);
					}
					else{
						MazeArray[PCol][PRow] = PUSHER;
						PutImg(PCol * BLOCK_WIDTH, PRow * BLOCK_HEIGHT + ROW_DISP, FORCE_RULE, Pusher);
					}

					break;
                case 41:
					if(MazeArray[PCol][PRow] == PUSHER_SAFE){
						MazeArray[PCol][PRow] = BLOCK_SAFE;
						PutImg(PCol * BLOCK_WIDTH, PRow * BLOCK_HEIGHT + ROW_DISP, FORCE_RULE, BlockSafe);
					}
					else{
						MazeArray[PCol][PRow] = BLOCK;
						PutImg(PCol * BLOCK_WIDTH, PRow * BLOCK_HEIGHT + ROW_DISP, FORCE_RULE, Block);
					}

					if(MazeArray[PCol + 1][PRow] == BLOCK_SAFE){
                        MazeArray[PCol + 1][PRow] = SAFE_SPOT;
						PutImg((PCol + 1) * BLOCK_WIDTH, PRow * BLOCK_HEIGHT + ROW_DISP, FORCE_RULE, SafeSpot);
                    }
					else{
						MazeArray[PCol + 1][PRow] = BLANK_SPOT;
                        PutImg((PCol + 1) * BLOCK_WIDTH, PRow * BLOCK_HEIGHT + ROW_DISP, FORCE_RULE, BlankSpot);
                    }

					PCol--;

					if(MazeArray[PCol][PRow] == SAFE_SPOT){
						MazeArray[PCol][PRow] = PUSHER_SAFE;
						PutImg(PCol * BLOCK_WIDTH, PRow * BLOCK_HEIGHT + ROW_DISP, FORCE_RULE, PusherSafe);
                    }
					else{
                        MazeArray[PCol][PRow] = PUSHER;
						PutImg(PCol * BLOCK_WIDTH, PRow * BLOCK_HEIGHT + ROW_DISP, FORCE_RULE, Pusher);
					}

					break;
            }

			last_move = 0;

			break;

		case DOWN_KEY:

            legal_move = (MazeArray[PCol][PRow + 1] != WALL_BLOCK);

			if((MazeArray[PCol][PRow + 1] < WALL_BLOCK) && (MazeArray[PCol][PRow + 2] < 4))
                legal_move = FALSE;

			if(legal_move){

				last_move = 10;

				if(MazeArray[PCol][PRow] == PUSHER_SAFE){
                    PutImg(PCol * BLOCK_WIDTH, PRow * BLOCK_HEIGHT + ROW_DISP, FORCE_RULE, SafeSpot);
                    MazeArray[PCol][PRow] = SAFE_SPOT;
				}
				else{
                    PutImg(PCol * BLOCK_WIDTH, PRow * BLOCK_HEIGHT + ROW_DISP, FORCE_RULE, BlankSpot);
                    MazeArray[PCol][PRow] = BLANK_SPOT;
                }

				PRow++;


				if(MazeArray[PCol][PRow] == BLOCK || MazeArray[PCol][PRow] == BLOCK_SAFE){

                    last_move++;

					if(MazeArray[PCol][PRow] == BLOCK && MazeArray[PCol][PRow + 1] == SAFE_SPOT){
						NumSafe++;
					}
					else if(MazeArray[PCol][PRow] == BLOCK_SAFE && MazeArray[PCol][PRow + 1] == BLANK_SPOT){
						NumSafe--;
					}

					if(MazeArray[PCol][PRow + 1] == SAFE_SPOT){
						MazeArray[PCol][PRow + 1] = BLOCK_SAFE;
						PutImg(PCol * BLOCK_WIDTH, (PRow + 1) * BLOCK_HEIGHT + ROW_DISP, FORCE_RULE, BlockSafe);

                        if(MazeArray[PCol][PRow] == BLOCK_SAFE){
							MazeArray[PCol][PRow] = SAFE_SPOT;
						}
                        else{
                            MazeArray[PCol][PRow] = BLANK_SPOT;
                        }
					}
					else{
						MazeArray[PCol][PRow + 1] = BLOCK;
						PutImg(PCol * BLOCK_WIDTH, (PRow + 1) * BLOCK_HEIGHT + ROW_DISP, FORCE_RULE, Block);

                        if(MazeArray[PCol][PRow] == BLOCK_SAFE){
							MazeArray[PCol][PRow] = SAFE_SPOT;
                        }
						else{
							MazeArray[PCol][PRow] = BLANK_SPOT;
						}

				    }

				}

				if(MazeArray[PCol][PRow] == SAFE_SPOT || MazeArray[PCol][PRow] == BLOCK_SAFE){

                    MazeArray[PCol][PRow] = PUSHER_SAFE;
                    PutImg(PCol * BLOCK_WIDTH, PRow * BLOCK_HEIGHT + ROW_DISP, FORCE_RULE, PusherSafe);
				}
				else{
					MazeArray[PCol][PRow] = PUSHER;
                    PutImg(PCol * BLOCK_WIDTH, PRow * BLOCK_HEIGHT + ROW_DISP, FORCE_RULE, Pusher);

                }
			}

			break;
		case UP_KEY:

			legal_move = (MazeArray[PCol][PRow - 1] != WALL_BLOCK);

            if((MazeArray[PCol][PRow - 1] < WALL_BLOCK) && (MazeArray[PCol][PRow - 2] < 4))
				legal_move = FALSE;

			if(legal_move){

				last_move = 20;

				if(MazeArray[PCol][PRow] == PUSHER_SAFE){
					PutImg(PCol * BLOCK_WIDTH, PRow * BLOCK_HEIGHT + ROW_DISP, FORCE_RULE, SafeSpot);
                    MazeArray[PCol][PRow] = SAFE_SPOT;
                }
				else{
					PutImg(PCol * BLOCK_WIDTH, PRow * BLOCK_HEIGHT + ROW_DISP, FORCE_RULE, BlankSpot);
					MazeArray[PCol][PRow] = BLANK_SPOT;
                }

                PRow--;

				if(MazeArray[PCol][PRow] == BLOCK || MazeArray[PCol][PRow] == BLOCK_SAFE){

					last_move++;

                    if(MazeArray[PCol][PRow] == BLOCK && MazeArray[PCol][PRow - 1] == SAFE_SPOT){
                        NumSafe++;
                    }
					else if(MazeArray[PCol][PRow] == BLOCK_SAFE && MazeArray[PCol][PRow - 1] == BLANK_SPOT){
						NumSafe--;
					}

					if(MazeArray[PCol][PRow - 1] == SAFE_SPOT){
						MazeArray[PCol][PRow - 1] = BLOCK_SAFE;
						PutImg(PCol * BLOCK_WIDTH, (PRow - 1) * BLOCK_HEIGHT + ROW_DISP, FORCE_RULE, BlockSafe);

						if(MazeArray[PCol][PRow] == BLOCK_SAFE){
							MazeArray[PCol][PRow] = SAFE_SPOT;
                        }
                        else{
                            MazeArray[PCol][PRow] = BLANK_SPOT;
						}
					}
					else{
						MazeArray[PCol][PRow- 1] = BLOCK;
					    PutImg(PCol * BLOCK_WIDTH, (PRow - 1) * BLOCK_HEIGHT + ROW_DISP, FORCE_RULE, Block);

						if(MazeArray[PCol][PRow] == BLOCK_SAFE){
							MazeArray[PCol][PRow] = SAFE_SPOT;
						}
						else{
                            MazeArray[PCol][PRow] = BLANK_SPOT;
                        }

					}

                }

			    if(MazeArray[PCol][PRow] == SAFE_SPOT || MazeArray[PCol][PRow] == BLOCK_SAFE){

					MazeArray[PCol][PRow] = PUSHER_SAFE;
					PutImg(PCol * BLOCK_WIDTH, PRow * BLOCK_HEIGHT + ROW_DISP, FORCE_RULE, PusherSafe);
				}
				else{
                    MazeArray[PCol][PRow] = PUSHER;
                    PutImg(PCol * BLOCK_WIDTH, PRow * BLOCK_HEIGHT + ROW_DISP, FORCE_RULE, Pusher);

				}
			}

			break;
		case LEFT_KEY:

			legal_move = (MazeArray[PCol - 1][PRow] != WALL_BLOCK);

			if((MazeArray[PCol - 1][PRow] < WALL_BLOCK) && (MazeArray[PCol - 2][PRow] < 4))
				legal_move = FALSE;

			if(legal_move){

				last_move = 30;

				if(MazeArray[PCol][PRow] == PUSHER_SAFE){
					PutImg(PCol * BLOCK_WIDTH, PRow * BLOCK_HEIGHT + ROW_DISP, FORCE_RULE, SafeSpot);
					MazeArray[PCol][PRow] = SAFE_SPOT;
				}
				else{
					PutImg(PCol * BLOCK_WIDTH, PRow * BLOCK_HEIGHT + ROW_DISP, FORCE_RULE, BlankSpot);
					MazeArray[PCol][PRow] = BLANK_SPOT;
                }

				PCol--;

			    if(MazeArray[PCol][PRow] == BLOCK || MazeArray[PCol][PRow] == BLOCK_SAFE){

                    last_move++;

					if(MazeArray[PCol][PRow] == BLOCK && MazeArray[PCol - 1][PRow] == SAFE_SPOT){
						NumSafe++;
					}
					else if(MazeArray[PCol][PRow] == BLOCK_SAFE && MazeArray[PCol - 1][PRow] == BLANK_SPOT){
						NumSafe--;
					}



					if(MazeArray[PCol - 1][PRow] == SAFE_SPOT){
					    MazeArray[PCol - 1][PRow] = BLOCK_SAFE;
						PutImg((PCol - 1) * BLOCK_WIDTH, PRow * BLOCK_HEIGHT + ROW_DISP, FORCE_RULE, BlockSafe);

						if(MazeArray[PCol][PRow] == BLOCK_SAFE){
							MazeArray[PCol][PRow] = SAFE_SPOT;
						}
						else{
                            MazeArray[PCol][PRow] = BLANK_SPOT;
                        }
                    }
					else{
						MazeArray[PCol - 1][PRow] = BLOCK;
					    PutImg((PCol - 1) * BLOCK_WIDTH, PRow * BLOCK_HEIGHT + ROW_DISP, FORCE_RULE, Block);

						if(MazeArray[PCol][PRow] == BLOCK_SAFE){
							MazeArray[PCol][PRow] = SAFE_SPOT;
						}
						else{
							MazeArray[PCol][PRow] = BLANK_SPOT;
						}

					}

				}

				if(MazeArray[PCol][PRow] == SAFE_SPOT || MazeArray[PCol][PRow] == BLOCK_SAFE){

                    MazeArray[PCol][PRow] = PUSHER_SAFE;
					PutImg(PCol * BLOCK_WIDTH, PRow * BLOCK_HEIGHT + ROW_DISP, FORCE_RULE, PusherSafe);
				}
				else{
					MazeArray[PCol][PRow] = PUSHER;
					PutImg(PCol * BLOCK_WIDTH, PRow * BLOCK_HEIGHT + ROW_DISP, FORCE_RULE, Pusher);

				}
            }

			break;
		case RIGHT_KEY:

            legal_move = (MazeArray[PCol + 1][PRow] != WALL_BLOCK);

			if((MazeArray[PCol + 1][PRow] < WALL_BLOCK) && (MazeArray[PCol + 2][PRow] < 4))
				legal_move = FALSE;

			if(legal_move){

                last_move = 40;

				if(MazeArray[PCol][PRow] == PUSHER_SAFE){
                    PutImg(PCol * BLOCK_WIDTH, PRow * BLOCK_HEIGHT + ROW_DISP, FORCE_RULE, SafeSpot);
                    MazeArray[PCol][PRow] = SAFE_SPOT;
				}
				else{
					PutImg(PCol * BLOCK_WIDTH, PRow * BLOCK_HEIGHT + ROW_DISP, FORCE_RULE, BlankSpot);
					MazeArray[PCol][PRow] = BLANK_SPOT;
				}

                PCol++;

				if(MazeArray[PCol][PRow] == BLOCK || MazeArray[PCol][PRow] == BLOCK_SAFE){

                    last_move++;

                    if(MazeArray[PCol][PRow] == BLOCK && MazeArray[PCol + 1][PRow] == SAFE_SPOT){
						NumSafe++;
					}
					else if(MazeArray[PCol][PRow] == BLOCK_SAFE && MazeArray[PCol + 1][PRow] == BLANK_SPOT){
						NumSafe--;
					}

				    if(MazeArray[PCol + 1][PRow] == SAFE_SPOT){
						MazeArray[PCol + 1][PRow] = BLOCK_SAFE;
						PutImg((PCol + 1) * BLOCK_WIDTH, PRow * BLOCK_HEIGHT + ROW_DISP, FORCE_RULE, BlockSafe);

						if(MazeArray[PCol][PRow] == BLOCK_SAFE){
                            MazeArray[PCol][PRow] = SAFE_SPOT;
						}
						else{
							MazeArray[PCol][PRow] = BLANK_SPOT;
						}
					}
					else{
						MazeArray[PCol + 1][PRow] = BLOCK;
					    PutImg((PCol + 1) * BLOCK_WIDTH, PRow * BLOCK_HEIGHT + ROW_DISP, FORCE_RULE, Block);

						if(MazeArray[PCol][PRow] == BLOCK_SAFE){
							MazeArray[PCol][PRow] = SAFE_SPOT;
                        }
                        else{
                            MazeArray[PCol][PRow] = BLANK_SPOT;
						}

					}

				}

				if(MazeArray[PCol][PRow] == SAFE_SPOT || MazeArray[PCol][PRow] == BLOCK_SAFE){

					MazeArray[PCol][PRow] = PUSHER_SAFE;
					PutImg(PCol * BLOCK_WIDTH, PRow * BLOCK_HEIGHT + ROW_DISP, FORCE_RULE, PusherSafe);
				}
                else{
					MazeArray[PCol][PRow] = PUSHER;
					PutImg(PCol * BLOCK_WIDTH, PRow * BLOCK_HEIGHT + ROW_DISP, FORCE_RULE, Pusher);

				}
			}

			break;
	}

	if(NumSafe == Blocks){
		Win(NOT_FINISHED);
    }

}


/*
 *	display help
 */
void ShowHelp(void)
{
	PALWIN	*pWin;

	pWin = OpenWin(WS_HELP, 50, 35, 590, 165, "OnLine Help");
	TextOut(70, 55, TXT_RULE, MEDIUM_FONT, "The object of the game is to move");
	TextOut(70, 70, TXT_RULE, MEDIUM_FONT, "all of the objects onto the dump.");
	TextOut(70, 85, TXT_RULE, MEDIUM_FONT, "You can only push one object at a");
	TextOut(70, 100, TXT_RULE, MEDIUM_FONT, "time and you cannot pull an object");
	TextOut(70, 115, TXT_RULE, MEDIUM_FONT, "Undo will only undo your last move.");
	TextOut(70, 140, TXT_RULE, SMALL_FONT, "Press any key to continue...");

	Line(425,40,425,160);
	Line(426,40,426,160);
	Line(427,40,427,160);
	Line(428,40,428,160);
	Line(429,40,429,160);
	Line(430,40,430,160);

	PutImg(438, 55, FORCE_RULE, Block);
	PutImg(438, 70, FORCE_RULE, BlockSafe);
	PutImg(438, 85, FORCE_RULE, SafeSpot);
	PutImg(438, 100, FORCE_RULE, Pusher);
	PutImg(438, 115, FORCE_RULE, PusherSafe);

	TextOut(455, 59, TXT_RULE, SMALL_FONT, "Object");
	TextOut(455, 74, TXT_RULE, SMALL_FONT, "Object on dump");
	TextOut(455, 89, TXT_RULE, SMALL_FONT, "Dump");
	TextOut(455, 104, TXT_RULE, SMALL_FONT, "You");
	TextOut(455, 119, TXT_RULE, SMALL_FONT, "You on dump");

	TextOut(465, 140, TXT_RULE, SMALL_FONT, "GOOD LUCK!");

	GetKey();

	CloseWin(pWin);
}


/*
 *	toggle the current level's "done" status
 */
void ToggleDone(void)
{
	fseek(fp, 0L, SEEK_SET);
	fread(SokDisk, sizeof(SokDisk), 1, fp);
	fseek(fp, 0L, SEEK_SET);

	if(SokDisk[level] != DONE)
		SokDisk[level] = DONE;
	else
		SokDisk[level] = NOT_DONE;


	fwrite(&SokDisk, sizeof(SokDisk), 1, fp);

	NextLevel(NEXT_DONE);
}


/*
 *	collect and handle key presses
 */
void Interact(void)
{
	int i;
	WORD Key;

   do {
		Key = GetKey();
	  if(Key & 0xff) Key &= 0xff;

		if(Key == MENU_KEY || Key == AF10_KEY) {
		 Key = HandleMenu(&BarMenu, 0, 0);
	  }
		switch(Key) {
			case DOWN_KEY:
				MoveChar(DOWN_KEY);
				break;
			case UP_KEY:
				MoveChar(UP_KEY);
				break;
			case LEFT_KEY:
				MoveChar(LEFT_KEY);
				break;
			case RIGHT_KEY:
				MoveChar(RIGHT_KEY);
				break;
			case F1_KEY:						/* show program help */
				ShowHelp();
				break;
			case F2_KEY:
				NextLevel(RESTART);
				break;
			case F3_KEY:
				MoveChar(UNDO);
				break;
			case F4_KEY:
				NextLevel(PREV);
				break;
			case F5_KEY:
				NextLevel(NEXT);
				break;
			case F6_KEY:
				NextLevel(SELECT);
				break;
			case F7_KEY:
				ToggleDone();
				break;
			case CMD_ABOUT:					/* display info about the game */
				AboutProg("HP-SOK", "Version 2.1a", COPY);
				break;
		}

	} while(Key != F8_KEY);
}



int main(void)
{
	if(!PalInit(1)) FatalExit("Init failed - CGAGRAPH not loaded ?", 1);

	if((fp = fopen("NEW-LEV.SOK", "rb+")) == NULL){
		if((fp = fopen("HP-SOK.SOK", "rb+")) == NULL){
			FatalExit("Cannot open game file, HP-SOK.SOK .", 1);
		}
	}

	PutImg(480, 18, FORCE_RULE, SokPCX);
	TextOut(570, 30, TXT_RULE, MEDIUM_FONT, "LEVEL");

	SetColor(BLACK_COLOR);
	SetRule(FORCE_RULE);
	Rectangle(481, 0, MAX_X-1, 1+FNTD(SMALL_FONT), SOLID_FILL);

	SetColor(WHITE_COLOR);

	TextOut(515, 1, TXT_RULE, SMALL_FONT, "HP-SOK v2.1a");

	AnnuncPos(AN_LEFT);
	ShowFKeys(KeyLabels);

	NextLevel(NEXT_DONE);

	Interact();

	fclose(fp);

	PalDeInit(1);

	return(0);
}

